#include "driver.h"

#include "trace.h"

#if defined(EVENT_TRACING)
#include "driver.tmh"
#endif


#ifdef ALLOC_PRAGMA
#pragma alloc_text (INIT, DriverEntry)
#pragma alloc_text (PAGE, ElveesEvtDriverContextCleanup)
#pragma alloc_text (PAGE, ElveesEvtDeviceAdd)
#endif


NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT  DriverObject,
    IN PUNICODE_STRING RegistryPath
    )
/*++

Routine Description:
    DriverEntry initializes the driver and is the first routine called by the
    system after the driver is loaded. DriverEntry specifies the other entry
    points in the function driver, such as EvtDevice and DriverUnload.

Parameters Description:

    DriverObject - represents the instance of the function driver that is loaded
    into memory. DriverEntry must initialize members of DriverObject before it
    returns to the caller. DriverObject is allocated by the system before the
    driver is loaded, and it is released by the system after the system unloads
    the function driver from memory.

    RegistryPath - represents the driver specific path in the Registry.
    The function driver can use the path to store driver related data between
    reboots. The path does not store hardware instance specific data.

Return Value:

    STATUS_SUCCESS if successful,
    STATUS_UNSUCCESSFUL otherwise.

--*/
{
    WDF_DRIVER_CONFIG config;
	WDF_OBJECT_ATTRIBUTES  attrib;
    NTSTATUS status;

	WPP_INIT_TRACING(DriverObject, RegistryPath);

	TraceEvents(TRACE_LEVEL_INFORMATION, DBG_INIT, "-->ElveesDriverEntry\n");

	WDF_OBJECT_ATTRIBUTES_INIT(&attrib);

	//
	// Register a cleanup callback so that we can call WPP_CLEANUP when
	// the framework driver object is deleted during driver unload.
	//
	attrib.EvtCleanupCallback = ElveesEvtDriverContextCleanup;

    WDF_DRIVER_CONFIG_INIT(&config,
                        ElveesEvtDeviceAdd
                        );

    status = WdfDriverCreate(DriverObject,
                            RegistryPath,
                            &attrib,
                            &config,
                            WDF_NO_HANDLE);
    if (!NT_SUCCESS(status)) {
		TraceEvents(TRACE_LEVEL_ERROR, DBG_INIT,
			"WdfDriverCreate failed with status %!STATUS!\n", status);
		//
		// Cleanup tracing here because DriverContextCleanup will not be called
		// as we have failed to create WDFDRIVER object itself.
		// Please note that if your return failure from DriverEntry after the
		// WDFDRIVER object is created successfully, you don't have to
		// call WPP cleanup because in those cases DriverContextCleanup
		// will be executed when the framework deletes the DriverObject.
		//
		WPP_CLEANUP(DriverObject);
		return status;
    }

	TraceEvents(TRACE_LEVEL_INFORMATION, DBG_INIT, "<--ElveesDriverEntry\n");

    return status;
}

NTSTATUS
ElveesEvtDeviceAdd(
    IN WDFDRIVER       Driver,
    IN PWDFDEVICE_INIT DeviceInit
    )
/*++
Routine Description:

    EvtDeviceAdd is called by the framework in response to AddDevice
    call from the PnP manager. We create and initialize a device object to
    represent a new instance of the device.

Arguments:

    Driver - Handle to a framework driver object created in DriverEntry

    DeviceInit - Pointer to a framework-allocated WDFDEVICE_INIT structure.

Return Value:

    NTSTATUS

--*/
{
    NTSTATUS status;

	TraceEvents(TRACE_LEVEL_INFORMATION, DBG_INIT, "-->ElveesEvtDeviceAdd\n");

    UNREFERENCED_PARAMETER(Driver);

    PAGED_CODE();

	// Since we are the function driver, we are now the power policy owner
	// for the device according to the default framework rule. We will register
	// our power policy callbacks after finding the wakeup capability of the device.

    status = ElveesDeviceCreate(DeviceInit);

	TraceEvents(TRACE_LEVEL_INFORMATION, DBG_INIT, "<--ElveesEvtDeviceAdd, status %!STATUS!\n", status);

    return status;
}

VOID
ElveesEvtDriverContextCleanup(IN WDFDRIVER Driver)
{
	PAGED_CODE();
	TraceEvents(TRACE_LEVEL_INFORMATION, DBG_INIT, "--ElveesEvtDriverContextCleanup\n");
	WPP_CLEANUP(WdfDriverWdmGetDriverObject(Driver));
}
