#include <QCoreApplication>
#include <QDebug>
#include <QTime>
#include <stdint.h>
#include <string.h>
#include "libpe.h"

#ifdef __linux
#include <unistd.h>
#define Sleep(x) usleep(x)
#endif

#define MCB_VEND_ID 0x680C2001

#define MAX_BLOCK_SIZE 65536

uint8_t buf[MAX_BLOCK_SIZE + 4];
unsigned block_size = 4096;

void mcb_close_and_exit()
{
    qDebug() << "\nExiting... Press ENTER...";

    libpe_close();

    getchar();
    exit(1);
}

void fatal_error()
{
#ifdef __linux
    qDebug() << "\nDRIVER INPUT/OUTPUT ERROR!";
#else
    qDebug() << "\nDRIVER INPUT/OUTPUT ERROR:" << GetLastError() << "!";
#endif
    mcb_close_and_exit();
}

uint32_t mcb_read_reg (uint32_t addr)
{
    uint32_t buf[2];
    buf[0] = addr;
    if (!libpe_read(buf, sizeof(buf)))
        fatal_error();

    return buf[1];
}

void mcb_write_reg(uint32_t addr, uint32_t value)
{
    uint32_t buf[2];

    buf[0] = addr;
    buf[1] = value;

    if (!libpe_write(buf, sizeof(buf)))
        fatal_error();
}

void test()
{
    qDebug("Testing DPRAM using %d byte blocks", block_size);

    unsigned nerr = 0;

    unsigned i;
    for (i = 0; i < block_size; ++i)
        buf[i + 4] = i;

    unsigned overall = 0;
    QTime timer;

    timer.start();

    while (overall + block_size < MCB_DPRAM_SIZE * 4) {
        uint32_t *paddr = (uint32_t *) buf;
        *paddr = MCB_DPRAM_BASE(0) + overall;
        libpe_write(buf, block_size + 4);
        overall += block_size;
    }

    int elapsed_ms = timer.elapsed();
    qDebug("Writing to DPRAM finished, %d ms elapsed, speed rate %.02f kbytes/sec",
           elapsed_ms, (float)overall / elapsed_ms);

    overall = 0;
    timer.start();
    while (overall + block_size < MCB_DPRAM_SIZE * 4) {
        //memset(buf, 0, sizeof(buf));
        uint32_t *paddr = (uint32_t *) buf;
        *paddr = MCB_DPRAM_BASE(0) + overall;
        libpe_read(buf, block_size + 4);
        for (i = 0; i < block_size; ++i)
            if (buf[i + 4] != (uint8_t) i) {
                qDebug("Bad value at address %05X, expected %02X, got %02X\n",
                       overall + i, (uint8_t) i, buf[i + 4]);
                nerr++;
            }
        overall += block_size;
    }

    elapsed_ms = timer.elapsed();
    qDebug("Reading from DPRAM finished, %d ms elapsed, speed rate %.02f kbytes/sec",
           elapsed_ms, (float)overall / elapsed_ms);

    qDebug() << "\nTest finished! Number of errors:" << nerr;
}

void usage()
{
    qDebug() << "Usage:";
    qDebug() << "    test-mem [block size]";
    qDebug() << "Block size is set in bytes. Default value: 4096 bytes.";
    exit(2);
}

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    qDebug() << "\n=== MCB-03PEM-PCI MEMORY TEST ===\n";

    if (a.arguments().count() == 1)
        ;
    else if (a.arguments().count() == 2)
        block_size = a.arguments()[1].toInt();
    else
        usage();

    if (block_size == 0 || block_size > MAX_BLOCK_SIZE) {
        qDebug() << "Unsupported block size!";
        usage();
    }

    if (!libpe_init()) {
        perror("Failed to initialize libmcb");
        exit(1);
    }

    qDebug("VENDOR ID/DEVICE ID: %08X\n",   mcb_read_reg(MCB_PCI_DEVICE_VENDOR_ID));
    qDebug("SWIC HW VERSION:     %08X\n\n", mcb_read_reg(MCB_SWIC_HW_VER(0)));

    test();

    mcb_close_and_exit();

    return 0;
}
